'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureStorageDriver = void 0;
const standalone_1 = require("@adonisjs/core/build/standalone");
const identity_1 = require("@azure/identity");
const storage_blob_1 = require("@azure/storage-blob");
const utils_1 = require("./utils");
class AzureStorageDriver {
    constructor(config) {
        this.config = config;
        /**
         * Name of the driver
         */
        this.name = 'AzureStorage';
        if (typeof config.connection_string !== 'undefined') {
            // eslint-disable-next-line
            this.adapter = storage_blob_1.BlobServiceClient.fromConnectionString(config.connection_string);
        }
        else {
            let credential;
            if (config.azure_tenant_id && config.azure_client_id && config.azure_client_secret) {
                credential = new identity_1.DefaultAzureCredential();
            }
            else if (config.name && config.key) {
                credential = new storage_blob_1.StorageSharedKeyCredential(config.name, config.key);
            }
            let url = `https://${this.config.name}.blob.core.windows.net`;
            if (typeof this.config.local_address !== 'undefined') {
                url = this.config.local_address;
            }
            const azurePipeline = (0, storage_blob_1.newPipeline)(credential);
            this.adapter = new storage_blob_1.BlobServiceClient(url, azurePipeline);
        }
    }
    getBlockBlobClient(location) {
        const container = this.config.container;
        const containerClient = this.adapter.getContainerClient(container);
        return containerClient.getBlockBlobClient(location);
    }
    async generateBlobSASURL(blockBlobClient, options) {
        options.permissions =
            options.permissions === undefined || typeof options.permissions === 'string'
                ? storage_blob_1.BlobSASPermissions.parse(options.permissions || 'r')
                : options.permissions;
        options.startsOn = options.startsOn || new Date();
        options.expiresOn = options.expiresOn || new Date(options.startsOn.valueOf() + 3600 * 1000);
        const factories = blockBlobClient.pipeline.factories;
        const credential = factories[factories.length - 1];
        const blobSAS = await (0, storage_blob_1.generateBlobSASQueryParameters)({
            containerName: blockBlobClient.containerName,
            blobName: blockBlobClient.name,
            permissions: options.permissions,
            startsOn: options.startsOn,
            expiresOn: options.expiresOn,
        }, credential);
        return `${blockBlobClient.url}?${blobSAS.toString()}`;
    }
    async existsContainer(container, options) {
        const containerClient = this.adapter.getContainerClient(container);
        try {
            return await containerClient.exists(options);
        }
        catch (error) {
            throw utils_1.CannotFindContainerException.invoke(container, error);
        }
    }
    /**
     * Creates a new container under the specified account. If the container with the same name already exists, the operation fails.
     * @param container A container name
     * @param options Options to Container Create operation.
     * @returns
     */
    async createContainer(container, options) {
        const containerClient = this.adapter.getContainerClient(container);
        try {
            return await containerClient.create(options);
        }
        catch (error) {
            throw utils_1.CannotCreateContainerException.invoke(container, error);
        }
    }
    /**
     * Marks the specified container for deletion. The container and any blobs contained within it are later deleted during garbage collection.
     * @param container A container name
     * @param options Options to Container Delete operation.
     * @returns
     */
    async deleteContainer(container, options) {
        const containerClient = this.adapter.getContainerClient(container);
        try {
            return await containerClient.delete(options);
        }
        catch (error) {
            throw utils_1.CannotDeleteContainerException.invoke(container, error);
        }
    }
    /**
     * Returns the file contents as a buffer. The buffer return
     * value allows you to self choose the encoding when
     * converting the buffer to a string.
     */
    async get(location, options = {}) {
        try {
            const blockBlobClient = this.getBlockBlobClient(location);
            return await blockBlobClient.downloadToBuffer(0, 0, options);
        }
        catch (error) {
            throw standalone_1.CannotReadFileException.invoke(location, error);
        }
    }
    /**
     * Returns the file contents as a stream
     */
    async getStream(location, options = {}) {
        return (await this.getBlockBlobClient(location).download(0, 0, options))
            .readableStreamBody;
    }
    /**
     * A boolean to find if the location path exists or not
     */
    exists(location, options = {}) {
        try {
            return this.getBlockBlobClient(location).exists(options);
        }
        catch (error) {
            throw standalone_1.CannotGetMetaDataException.invoke(location, 'exists', error);
        }
    }
    /**
     * Returns the signed url for a given path
     */
    async getSignedUrl(location, options) {
        options = options || {
            containerName: this.config.container,
        };
        options.containerName = options.containerName || this.config.container;
        try {
            const blockBlobClient = this.getBlockBlobClient(location);
            const SASUrl = await this.generateBlobSASURL(blockBlobClient, options);
            return SASUrl;
        }
        catch (error) {
            throw standalone_1.CannotGetMetaDataException.invoke(location, 'signedUrl', error);
        }
    }
    /**
     * Returns URL to a given path
     */
    async getUrl(location) {
        return unescape(this.getBlockBlobClient(location).url);
    }
    /**
     * Write string|buffer contents to a destination. The missing
     * intermediate directories will be created (if required).
     *
     * @todo look into returning the response of upload
     */
    async put(location, contents, options) {
        const blockBlobClient = this.getBlockBlobClient(location);
        try {
            await blockBlobClient.upload(contents, contents.length, options);
        }
        catch (error) {
            throw standalone_1.CannotWriteFileException.invoke(location, error);
        }
    }
    /**
     * Write a stream to a destination. The missing intermediate
     * directories will be created (if required).
     */
    async putStream(location, contents, options) {
        const blockBlobClient = this.getBlockBlobClient(location);
        try {
            await blockBlobClient.uploadStream(contents, undefined, undefined, options);
        }
        catch (error) {
            console.log(error);
            throw standalone_1.CannotWriteFileException.invoke(location, error);
        }
    }
    /**
     * Copy a given location path from the source to the desination.
     * The missing intermediate directories will be created (if required)
     *
     * @todo look into returning the response of syncCopyFromURL
     */
    async copy(source, destination, options) {
        options = options || {
            containerName: this.config.container,
        };
        options.containerName = options.containerName || this.config.container;
        const sourceBlockBlobClient = this.getBlockBlobClient(source);
        const destinationBlockBlobClient = this.getBlockBlobClient(destination);
        const url = await this.generateBlobSASURL(sourceBlockBlobClient, options);
        try {
            await destinationBlockBlobClient.syncCopyFromURL(url);
        }
        catch (error) {
            throw standalone_1.CannotCopyFileException.invoke(source, destination, error.original || error);
        }
    }
    /**
     * Remove a given location path
     *
     * @todo find a way to extend delete with BlobDeleteOptions
     */
    async delete(location) {
        try {
            await this.getBlockBlobClient(location).delete();
        }
        catch (error) {
            throw standalone_1.CannotDeleteFileException.invoke(location, error);
        }
    }
    /**
     * Move a given location path from the source to the desination.
     * The missing intermediate directories will be created (if required)
     */
    async move(source, destination, options) {
        try {
            await this.copy(source, destination, options);
            await this.delete(source);
        }
        catch (error) {
            throw standalone_1.CannotMoveFileException.invoke(source, destination, error.original || error);
        }
    }
    /**
     * Returns the file stats
     */
    async getStats(location) {
        try {
            const metaData = await this.getBlockBlobClient(location).getProperties();
            return {
                modified: metaData.lastModified,
                size: metaData.contentLength,
                isFile: true,
                etag: metaData.etag,
            };
        }
        catch (error) {
            throw standalone_1.CannotGetMetaDataException.invoke(location, 'stats', error);
        }
    }
    /**
     * Not Supported
     *
     * Returns the file visibility
     *
     */
    async getVisibility(location) {
        throw standalone_1.CannotGetMetaDataException.invoke(location, 'visibility', 'Not Supported');
    }
    /**
     * Not supported
     *
     * Sets the file visibility
     */
    async setVisibility(location) {
        throw standalone_1.CannotSetVisibilityException.invoke(location, 'Not Supported');
    }
}
exports.AzureStorageDriver = AzureStorageDriver;
